@@ -1,4 +1,4 @@
-/* Copyright 2012--2016 The Tor Project
+/* Copyright 2012--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -7,204 +7,205 @@ import org.torproject.descriptor.BandwidthHistory;
 import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.ServerDescriptor;
 
-import java.io.UnsupportedEncodingException;
-import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashSet;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Scanner;
 import java.util.Set;
-
-import javax.xml.bind.DatatypeConverter;
+import java.util.SortedMap;
+import java.util.SortedSet;
 
 /* Contains a server descriptor. */
 public abstract class ServerDescriptorImpl extends DescriptorImpl
     implements ServerDescriptor {
 
-  protected ServerDescriptorImpl(byte[] descriptorBytes,
+  private static final Set<Key> atMostOnce = EnumSet.of(
+      Key.IDENTITY_ED25519, Key.MASTER_KEY_ED25519, Key.PLATFORM, Key.PROTO,
+      Key.FINGERPRINT, Key.HIBERNATING, Key.UPTIME, Key.CONTACT, Key.FAMILY,
+      Key.READ_HISTORY, Key.WRITE_HISTORY, Key.EVENTDNS, Key.CACHES_EXTRA_INFO,
+      Key.EXTRA_INFO_DIGEST, Key.HIDDEN_SERVICE_DIR, Key.PROTOCOLS,
+      Key.ALLOW_SINGLE_HOP_EXITS, Key.ONION_KEY, Key.SIGNING_KEY,
+      Key.IPV6_POLICY, Key.NTOR_ONION_KEY, Key.ONION_KEY_CROSSCERT,
+      Key.NTOR_ONION_KEY_CROSSCERT, Key.TUNNELLED_DIR_SERVER,
+      Key.ROUTER_SIG_ED25519, Key.ROUTER_SIGNATURE, Key.ROUTER_DIGEST_SHA256,
+      Key.ROUTER_DIGEST);
+
+  private static final Set<Key> exactlyOnce = EnumSet.of(
+      Key.ROUTER, Key.BANDWIDTH, Key.PUBLISHED);
+
+  protected ServerDescriptorImpl(byte[] descriptorBytes, int[] offsetAndLength,
       boolean failUnrecognizedDescriptorLines)
       throws DescriptorParseException {
-    super(descriptorBytes, failUnrecognizedDescriptorLines, false);
+    super(descriptorBytes, offsetAndLength, failUnrecognizedDescriptorLines,
+        false);
     this.parseDescriptorBytes();
-    this.calculateDigest();
-    this.calculateDigestSha256();
-    Set<String> exactlyOnceKeywords = new HashSet<>(Arrays.asList(
-        "router,bandwidth,published".split(",")));
-    this.checkExactlyOnceKeywords(exactlyOnceKeywords);
-    Set<String> atMostOnceKeywords = new HashSet<>(Arrays.asList((
-        "identity-ed25519,master-key-ed25519,platform,fingerprint,"
-        + "hibernating,uptime,contact,family,read-history,write-history,"
-        + "eventdns,caches-extra-info,extra-info-digest,"
-        + "hidden-service-dir,protocols,allow-single-hop-exits,onion-key,"
-        + "signing-key,ipv6-policy,ntor-onion-key,onion-key-crosscert,"
-        + "ntor-onion-key-crosscert,tunnelled-dir-server,"
-        + "router-sig-ed25519,router-signature,router-digest-sha256,"
-        + "router-digest").split(",")));
-    this.checkAtMostOnceKeywords(atMostOnceKeywords);
-    this.checkFirstKeyword("router");
-    if (this.getKeywordCount("accept") == 0
-        && this.getKeywordCount("reject") == 0) {
+    this.checkExactlyOnceKeys(exactlyOnce);
+    this.checkAtMostOnceKeys(atMostOnce);
+    this.checkFirstKey(Key.ROUTER);
+    if (this.getKeyCount(Key.ACCEPT) == 0
+        && this.getKeyCount(Key.REJECT) == 0) {
       throw new DescriptorParseException("Either keyword 'accept' or "
           + "'reject' must be contained at least once.");
     }
-    this.clearParsedKeywords();
+    this.clearParsedKeys();
     return;
   }
 
   private void parseDescriptorBytes() throws DescriptorParseException {
-    Scanner scanner = new Scanner(new String(this.rawDescriptorBytes))
-        .useDelimiter("\n");
-    String nextCrypto = "";
+    Scanner scanner = this.newScanner().useDelimiter(NL);
+    Key nextCrypto = Key.EMPTY;
     List<String> cryptoLines = null;
     while (scanner.hasNext()) {
       String line = scanner.next();
       if (line.startsWith("@")) {
         continue;
       }
-      String lineNoOpt = line.startsWith("opt ")
-          ? line.substring("opt ".length()) : line;
+      String lineNoOpt = line.startsWith(Key.OPT.keyword + SP)
+          ? line.substring(Key.OPT.keyword.length() + 1) : line;
       String[] partsNoOpt = lineNoOpt.split("[ \t]+");
-      String keyword = partsNoOpt[0];
-      switch (keyword) {
-        case "router":
+      Key key = Key.get(partsNoOpt[0]);
+      switch (key) {
+        case ROUTER:
           this.parseRouterLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "or-address":
+        case OR_ADDRESS:
           this.parseOrAddressLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "bandwidth":
+        case BANDWIDTH:
           this.parseBandwidthLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "platform":
+        case PLATFORM:
           this.parsePlatformLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "published":
+        case PROTO:
+          this.parseProtoLine(line, lineNoOpt, partsNoOpt);
+          break;
+        case PUBLISHED:
           this.parsePublishedLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "fingerprint":
+        case FINGERPRINT:
           this.parseFingerprintLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "hibernating":
+        case HIBERNATING:
           this.parseHibernatingLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "uptime":
+        case UPTIME:
           this.parseUptimeLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "onion-key":
+        case ONION_KEY:
           this.parseOnionKeyLine(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "onion-key";
+          nextCrypto = key;
           break;
-        case "signing-key":
+        case SIGNING_KEY:
           this.parseSigningKeyLine(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "signing-key";
+          nextCrypto = key;
           break;
-        case "accept":
+        case ACCEPT:
           this.parseAcceptLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "reject":
+        case REJECT:
           this.parseRejectLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "router-signature":
+        case ROUTER_SIGNATURE:
           this.parseRouterSignatureLine(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "router-signature";
+          nextCrypto = key;
           break;
-        case "contact":
+        case CONTACT:
           this.parseContactLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "family":
+        case FAMILY:
           this.parseFamilyLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "read-history":
+        case READ_HISTORY:
           this.parseReadHistoryLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "write-history":
+        case WRITE_HISTORY:
           this.parseWriteHistoryLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "eventdns":
+        case EVENTDNS:
           this.parseEventdnsLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "caches-extra-info":
+        case CACHES_EXTRA_INFO:
           this.parseCachesExtraInfoLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "extra-info-digest":
+        case EXTRA_INFO_DIGEST:
           this.parseExtraInfoDigestLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "hidden-service-dir":
+        case HIDDEN_SERVICE_DIR:
           this.parseHiddenServiceDirLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "protocols":
+        case PROTOCOLS:
           this.parseProtocolsLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "allow-single-hop-exits":
+        case ALLOW_SINGLE_HOP_EXITS:
           this.parseAllowSingleHopExitsLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "dircacheport":
+        case DIRCACHEPORT:
           this.parseDircacheportLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "router-digest":
+        case ROUTER_DIGEST:
           this.parseRouterDigestLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "router-digest-sha256":
+        case ROUTER_DIGEST_SHA256:
           this.parseRouterDigestSha256Line(line, lineNoOpt, partsNoOpt);
           break;
-        case "ipv6-policy":
+        case IPV6_POLICY:
           this.parseIpv6PolicyLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "ntor-onion-key":
+        case NTOR_ONION_KEY:
           this.parseNtorOnionKeyLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "identity-ed25519":
+        case IDENTITY_ED25519:
           this.parseIdentityEd25519Line(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "identity-ed25519";
+          nextCrypto = key;
           break;
-        case "master-key-ed25519":
+        case MASTER_KEY_ED25519:
           this.parseMasterKeyEd25519Line(line, lineNoOpt, partsNoOpt);
           break;
-        case "router-sig-ed25519":
+        case ROUTER_SIG_ED25519:
           this.parseRouterSigEd25519Line(line, lineNoOpt, partsNoOpt);
           break;
-        case "onion-key-crosscert":
+        case ONION_KEY_CROSSCERT:
           this.parseOnionKeyCrosscert(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "onion-key-crosscert";
+          nextCrypto = key;
           break;
-        case "ntor-onion-key-crosscert":
+        case NTOR_ONION_KEY_CROSSCERT:
           this.parseNtorOnionKeyCrosscert(line, lineNoOpt, partsNoOpt);
-          nextCrypto = "ntor-onion-key-crosscert";
+          nextCrypto = key;
           break;
-        case "tunnelled-dir-server":
+        case TUNNELLED_DIR_SERVER:
           this.parseTunnelledDirServerLine(line, lineNoOpt, partsNoOpt);
           break;
-        case "-----BEGIN":
+        case CRYPTO_BEGIN:
           cryptoLines = new ArrayList<>();
           cryptoLines.add(line);
           break;
-        case "-----END":
+        case CRYPTO_END:
           cryptoLines.add(line);
           StringBuilder sb = new StringBuilder();
           for (String cryptoLine : cryptoLines) {
-            sb.append("\n").append(cryptoLine);
+            sb.append(NL).append(cryptoLine);
           }
           String cryptoString = sb.toString().substring(1);
           switch (nextCrypto) {
-            case "onion-key":
+            case ONION_KEY:
               this.onionKey = cryptoString;
               break;
-            case "signing-key":
+            case SIGNING_KEY:
               this.signingKey = cryptoString;
               break;
-            case "router-signature":
+            case ROUTER_SIGNATURE:
               this.routerSignature = cryptoString;
               break;
-            case "identity-ed25519":
+            case IDENTITY_ED25519:
               this.identityEd25519 = cryptoString;
               this.parseIdentityEd25519CryptoBlock(cryptoString);
               break;
-            case "onion-key-crosscert":
+            case ONION_KEY_CROSSCERT:
               this.onionKeyCrosscert = cryptoString;
               break;
-            case "ntor-onion-key-crosscert":
+            case NTOR_ONION_KEY_CROSSCERT:
               this.ntorOnionKeyCrosscert = cryptoString;
               break;
             default:
@@ -219,8 +220,9 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
               }
           }
           cryptoLines = null;
-          nextCrypto = "";
+          nextCrypto = Key.EMPTY;
           break;
+        case INVALID:
         default:
           if (cryptoLines != null) {
             cryptoLines.add(line);
@@ -296,13 +298,19 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parsePlatformLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (lineNoOpt.length() > "platform ".length()) {
-      this.platform = lineNoOpt.substring("platform ".length());
+    if (lineNoOpt.length() > Key.PLATFORM.keyword.length() + 1) {
+      this.platform = lineNoOpt.substring(Key.PLATFORM.keyword.length() + 1);
     } else {
       this.platform = "";
     }
   }
 
+  private void parseProtoLine(String line, String lineNoOpt,
+      String[] partsNoOpt) throws DescriptorParseException {
+    this.protocols = ParseHelper.parseProtocolVersions(line, lineNoOpt,
+        partsNoOpt);
+  }
+
   private void parsePublishedLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
     this.publishedMillis = ParseHelper.parseTimestampAtIndex(line,
@@ -311,11 +319,12 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseFingerprintLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (lineNoOpt.length() != "fingerprint".length() + 5 * 10) {
+    if (lineNoOpt.length() != Key.FINGERPRINT.keyword.length() + 5 * 10) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
     this.fingerprint = ParseHelper.parseTwentyByteHexString(line,
-        lineNoOpt.substring("fingerprint ".length()).replaceAll(" ", ""));
+        lineNoOpt.substring(Key.FINGERPRINT.keyword.length() + 1)
+            .replaceAll(SP, ""));
   }
 
   private void parseHibernatingLine(String line, String lineNoOpt,
@@ -347,14 +356,14 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseOnionKeyLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("onion-key")) {
+    if (!lineNoOpt.equals(Key.ONION_KEY.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
   }
 
   private void parseSigningKeyLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("signing-key")) {
+    if (!lineNoOpt.equals(Key.SIGNING_KEY.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
   }
@@ -380,15 +389,15 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseRouterSignatureLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("router-signature")) {
+    if (!lineNoOpt.equals(Key.ROUTER_SIGNATURE.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
   }
 
   private void parseContactLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (lineNoOpt.length() > "contact ".length()) {
-      this.contact = lineNoOpt.substring("contact ".length());
+    if (lineNoOpt.length() > Key.CONTACT.keyword.length() + 1) {
+      this.contact = lineNoOpt.substring(Key.CONTACT.keyword.length() + 1);
     } else {
       this.contact = "";
     }
@@ -444,7 +453,7 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseCachesExtraInfoLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("caches-extra-info")) {
+    if (!lineNoOpt.equals(Key.CACHES_EXTRA_INFO.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
     this.cachesExtraInfo = true;
@@ -521,7 +530,7 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseAllowSingleHopExitsLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("allow-single-hop-exits")) {
+    if (!lineNoOpt.equals(Key.ALLOW_SINGLE_HOP_EXITS.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
     this.allowSingleHopExits = true;
@@ -547,8 +556,8 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
     if (partsNoOpt.length != 2) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
-    this.serverDescriptorDigest = ParseHelper.parseTwentyByteHexString(
-        line, partsNoOpt[1]);
+    this.setDigestSha1Hex(ParseHelper.parseTwentyByteHexString(
+        line, partsNoOpt[1]));
   }
 
   private void parseIpv6PolicyLine(String line, String lineNoOpt,
@@ -557,9 +566,9 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
     if (partsNoOpt.length != 3) {
       isValid = false;
     } else {
-      switch (partsNoOpt[1]) {
-        case "accept":
-        case "reject":
+      switch (Key.get(partsNoOpt[1])) {
+        case ACCEPT:
+        case REJECT:
           this.ipv6DefaultPolicy = partsNoOpt[1];
           this.ipv6PortList = partsNoOpt[2];
           String[] ports = partsNoOpt[2].split(",", -1);
@@ -570,6 +579,7 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
             }
           }
           break;
+        case INVALID:
         default:
           isValid = false;
       }
@@ -615,7 +625,7 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   private void parseTunnelledDirServerLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (!lineNoOpt.equals("tunnelled-dir-server")) {
+    if (!lineNoOpt.equals(Key.TUNNELLED_DIR_SERVER.keyword)) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
     this.tunnelledDirServer = true;
@@ -662,84 +672,17 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
       throw new DescriptorParseException("Illegal line '" + line + "'.");
     }
     ParseHelper.parseThirtyTwoByteBase64String(line, partsNoOpt[1]);
-    this.serverDescriptorDigestSha256 = partsNoOpt[1];
+    this.setDigestSha256Base64(partsNoOpt[1]);
   }
 
-  private void calculateDigest() throws DescriptorParseException {
-    if (this.serverDescriptorDigest != null) {
-      /* We already learned the descriptor digest of this bridge
-       * descriptor from a "router-digest" line. */
-      return;
-    }
-    try {
-      String ascii = new String(this.getRawDescriptorBytes(), "US-ASCII");
-      String startToken = "router ";
-      String sigToken = "\nrouter-signature\n";
-      int start = ascii.indexOf(startToken);
-      int sig = ascii.indexOf(sigToken) + sigToken.length();
-      if (start >= 0 && sig >= 0 && sig > start) {
-        byte[] forDigest = new byte[sig - start];
-        System.arraycopy(this.getRawDescriptorBytes(), start,
-            forDigest, 0, sig - start);
-        this.serverDescriptorDigest = DatatypeConverter.printHexBinary(
-            MessageDigest.getInstance("SHA-1").digest(forDigest))
-            .toLowerCase();
-      }
-    } catch (UnsupportedEncodingException e) {
-      /* Handle below. */
-    } catch (NoSuchAlgorithmException e) {
-      /* Handle below. */
-    }
-    if (this.serverDescriptorDigest == null) {
-      throw new DescriptorParseException("Could not calculate server "
-          + "descriptor digest.");
-    }
-  }
-
-  private void calculateDigestSha256() throws DescriptorParseException {
-    if (this.serverDescriptorDigestSha256 != null) {
-      /* We already learned the descriptor digest of this bridge
-       * descriptor from a "router-digest-sha256" line. */
-      return;
-    }
-    try {
-      String ascii = new String(this.getRawDescriptorBytes(), "US-ASCII");
-      String startToken = "router ";
-      String sigToken = "\n-----END SIGNATURE-----\n";
-      int start = ascii.indexOf(startToken);
-      int sig = ascii.indexOf(sigToken) + sigToken.length();
-      if (start >= 0 && sig >= 0 && sig > start) {
-        byte[] forDigest = new byte[sig - start];
-        System.arraycopy(this.getRawDescriptorBytes(), start, forDigest,
-            0, sig - start);
-        this.serverDescriptorDigestSha256 =
-            DatatypeConverter.printBase64Binary(
-            MessageDigest.getInstance("SHA-256").digest(forDigest))
-            .replaceAll("=", "");
-      }
-    } catch (UnsupportedEncodingException e) {
-      /* Handle below. */
-    } catch (NoSuchAlgorithmException e) {
-      /* Handle below. */
-    }
-    if (this.serverDescriptorDigestSha256 == null) {
-      throw new DescriptorParseException("Could not calculate server "
-          + "descriptor SHA-256 digest.");
-    }
-  }
-
-  private String serverDescriptorDigest;
-
   @Override
   public String getServerDescriptorDigest() {
-    return this.serverDescriptorDigest;
+    return this.getDigestSha1Hex();
   }
 
-  private String serverDescriptorDigestSha256;
-
   @Override
   public String getServerDescriptorDigestSha256() {
-    return this.serverDescriptorDigestSha256;
+    return this.getDigestSha256Base64();
   }
 
   private String nickname;
@@ -812,6 +755,13 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
     return this.platform;
   }
 
+  private SortedMap<String, SortedSet<Long>> protocols;
+
+  @Override
+  public SortedMap<String, SortedSet<Long>> getProtocols() {
+    return this.protocols;
+  }
+
   private long publishedMillis;
 
   @Override
@@ -915,6 +865,11 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getExtraInfoDigest() {
+    return this.getExtraInfoDigestSha1Hex();
+  }
+
+  @Override
+  public String getExtraInfoDigestSha1Hex() {
     return this.extraInfoDigest;
   }
 
@@ -922,6 +877,11 @@ public abstract class ServerDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getExtraInfoDigestSha256() {
+    return this.getExtraInfoDigestSha256Base64();
+  }
+
+  @Override
+  public String getExtraInfoDigestSha256Base64() {
     return this.extraInfoDigestSha256;
   }
 
